@@ -152,51 +152,51 @@ sub show_backup_destination
 {
 my ($mode, $user, $pass, $server, $path, $port) = &parse_backup_url($_[1]);
 my $rv;
-$rv .= "<table cellpadding=1 cellspacing=0>";
+$rv .= "<table id='show_backup_destination' cellpadding=1 cellspacing=0>";
 
 # Local file field
 $rv .= "<tr><td>".&ui_oneradio("$_[0]_mode", 0, undef, $mode == 0)."</td>\n";
-$rv .= "<td colspan=2>$text{'backup_mode0'} ".
-	&ui_textbox("$_[0]_file", $mode == 0 ? $path : "", 40).
+$rv .= "<td>$text{'backup_mode0'}&nbsp;</td><td colspan='3'>".
+	&ui_textbox("$_[0]_file", $mode == 0 ? $path : "", 60).
 	" ".&file_chooser_button("$_[0]_file")."</td> </tr>\n";
 
 # FTP file fields
 $rv .= "<tr><td>".&ui_oneradio("$_[0]_mode", 1, undef, $mode == 1)."</td>\n";
-$rv .= "<td>$text{'backup_mode1'} ".
+$rv .= "<td>$text{'backup_mode1'}&nbsp;</td><td>".
 	&ui_textbox("$_[0]_server", $mode == 1 ? $server : undef, 20).
 	"</td>\n";
-$rv .= "<td>$text{'backup_path'} ".
-	&ui_textbox("$_[0]_path", $mode == 1 ? $path : undef, 40).
+$rv .= "<td>&nbsp;$text{'backup_path'}&nbsp;</td><td> ".
+	&ui_textbox("$_[0]_path", $mode == 1 ? $path : undef, 20).
 	"</td> </tr>\n";
 $rv .= "<tr> <td></td>\n";
-$rv .= "<td>$text{'backup_login'} ".
-	&ui_textbox("$_[0]_user", $mode == 1 ? $user : undef, 15).
+$rv .= "<td>$text{'backup_login'}&nbsp;</td><td> ".
+	&ui_textbox("$_[0]_user", $mode == 1 ? $user : undef, 20).
 	"</td>\n";
-$rv .= "<td>$text{'backup_pass'} ".
-	&ui_password("$_[0]_pass", $mode == 1 ? $pass : undef, 15).
+$rv .= "<td>&nbsp;$text{'backup_pass'}&nbsp;</td><td> ".
+	&ui_password("$_[0]_pass", $mode == 1 ? $pass : undef, 20).
 	"</td> </tr>\n";
 $rv .= "<tr> <td></td>\n";
-$rv .= "<td>$text{'backup_port'} ".
+$rv .= "<td colspan='4'>$text{'backup_port'} ".
 	&ui_opt_textbox("$_[0]_port", $mode == 1 ? $port : undef, 5,
 			$text{'default'})."</td> </tr>\n";
 
 # SCP file fields
 $rv .= "<tr><td>".&ui_oneradio("$_[0]_mode", 2, undef, $mode == 2)."</td>\n";
-$rv .= "<td>$text{'backup_mode2'} ".
+$rv .= "<td>$text{'backup_mode2'}&nbsp;</td><td>".
 	&ui_textbox("$_[0]_sserver", $mode == 2 ? $server : undef, 20).
 	"</td>\n";
-$rv .= "<td>$text{'backup_path'} ".
-	&ui_textbox("$_[0]_spath", $mode == 2 ? $path : undef, 40).
+$rv .= "<td>&nbsp;$text{'backup_path'}&nbsp;</td><td> ".
+	&ui_textbox("$_[0]_spath", $mode == 2 ? $path : undef, 20).
 	"</td> </tr>\n";
 $rv .= "<tr> <td></td>\n";
-$rv .= "<td>$text{'backup_login'} ".
-	&ui_textbox("$_[0]_suser", $mode == 2 ? $user : undef, 15).
+$rv .= "<td>$text{'backup_login'}&nbsp;</td><td> ".
+	&ui_textbox("$_[0]_suser", $mode == 2 ? $user : undef, 20).
 	"</td>\n";
-$rv .= "<td>$text{'backup_pass'} ".
-	&ui_password("$_[0]_spass", $mode == 2 ? $pass : undef, 15).
+$rv .= "<td>&nbsp;$text{'backup_pass'}&nbsp;</td><td> ".
+	&ui_password("$_[0]_spass", $mode == 2 ? $pass : undef, 20).
 	"</td> </tr>\n";
 $rv .= "<tr> <td></td>\n";
-$rv .= "<td>$text{'backup_port'} ".
+$rv .= "<td colspan='4'>$text{'backup_port'} ".
 	&ui_opt_textbox("$_[0]_sport", $mode == 2 ? $port : undef, 5,
 			$text{'default'})."</td> </tr>\n";
 
@@ -204,7 +204,7 @@ if ($_[2] == 1) {
 	# Uploaded file field
 	$rv .= "<tr><td>".&ui_oneradio("$_[0]_mode", 3, undef, $mode == 3).
 		"</td>\n";
-	$rv .= "<td colspan=2>$text{'backup_mode3'} ".
+	$rv .= "<td colspan=4>$text{'backup_mode3'} ".
 		&ui_upload("$_[0]_upload", 40).
 		"</td> </tr>\n";
 	}
@@ -212,7 +212,7 @@ elsif ($_[2] == 2) {
 	# Output to browser option
 	$rv .= "<tr><td>".&ui_oneradio("$_[0]_mode", 4, undef, $mode == 4).
 		"</td>\n";
-	$rv .= "<td colspan=2>$text{'backup_mode4'}</td> </tr>\n";
+	$rv .= "<td colspan=4>$text{'backup_mode4'}</td> </tr>\n";
 	}
 
 $rv .= "</table>\n";
@@ -227,10 +227,11 @@ Returns a backup destination string, or calls error.
 sub parse_backup_destination
 {
 my %in = %{$_[1]};
-my $mode = $in{"$_[0]_mode"};
+my $mode = $in{"$_[0]_mode"} || 0;
 if ($mode == 0) {
 	# Local file
-	$in{"$_[0]_file"} =~ /^\/\S/ || &error($text{'backup_edest'});
+	$in{"$_[0]_file"} && $in{"$_[0]_file"} =~ /^\/\S/ ||
+		&error($text{'backup_edest'});
 	return $in{"$_[0]_file"};
 	}
 elsif ($mode == 1) {
@@ -298,7 +299,8 @@ if ($mode == 0) {
 	$file = &date_subs($path);
 	}
 else {
-	$file = &transname();
+	my $ext = &has_command("gzip") ? '.tar.gz' : '.tar';
+	$file = &transname_timestamped("backup", $ext);
 	}
 
 # Get module descriptions
@@ -335,10 +337,11 @@ if (!$_[5]) {
 		}
 	}
 
-# Add module config files
+# Add module config files and custom langs
 if ($_[4]) {
 	foreach $m (@mods) {
 		my @cfiles = ( "$config_directory/$m/config" );
+		push(@cfiles, glob("$config_directory/$m/custom-lang*"));
 		push(@files, @cfiles);
 		push(@{$manifestfiles{$m}}, @cfiles);
 		}
@@ -452,7 +455,8 @@ elsif ($mode == 2) {
 return undef;
 }
 
-=head2 execute_restore(&mods, source, &files, apply, [show-only])
+=head2 execute_restore(&mods, source, &files, apply, [show-only],
+		       [&other-files])
 
 Restore configuration files from the specified source for the listed modules.
 Returns undef on success, or an error message.
@@ -460,7 +464,7 @@ Returns undef on success, or an error message.
 =cut
 sub execute_restore
 {
-my ($mods, $src, $files, $apply, $show) = @_;
+my ($mods, $src, $files, $apply, $show, $others) = @_;
 
 # Fetch file if needed
 my ($mode, $user, $pass, $host, $path, $port) = &parse_backup_url($src);
@@ -492,7 +496,7 @@ else {
 	}
 
 # Validate archive
-open(FILE, $file);
+open(FILE, "<".$file);
 my $two;
 read(FILE, $two, 2);
 close(FILE);
@@ -517,7 +521,7 @@ my @tarfiles = map { "/$_" } split(/\r?\n/, $out);
 my %tarfiles = map { $_, 1 } @tarfiles;
 
 # Extract manifests for each module
-my %hasmod = map { $_, 1 } @{$_[0]};
+my %hasmod = map { $_, 1 } @$mods;
 $hasmod{"_others"} = 1;
 &execute_command("rm -rf ".quotemeta($manifests_dir));
 my $rel_manifests_dir = $manifests_dir;
@@ -534,7 +538,7 @@ my %mfiles;
 my @files;
 while($m = readdir(DIR)) {
 	next if ($m eq "." || $m eq ".." || !$hasmod{$m});
-	open(MAN, "$manifests_dir/$m");
+	open(MAN, "<$manifests_dir/$m");
 	my @mfiles;
 	while(<MAN>) {
 		s/\r|\n//g;
@@ -547,6 +551,7 @@ while($m = readdir(DIR)) {
 	push(@files, @mfiles);
 	}
 closedir(DIR);
+push(@files, @$others) if ($others);
 if (!@files) {
 	&unlink_file($file) if ($mode != 0);
 	return $text{'backup_enone2'};
@@ -554,15 +559,15 @@ if (!@files) {
 
 # Get descriptions for each module
 my %desc;
-foreach my $m (@{$_[0]}) {
+foreach my $m (@$mods) {
 	my %minfo = &get_module_info($m);
 	$desc{$m} = $minfo{'desc'};
 	}
 
 # Call module pre functions
-foreach my $m (@{$_[0]}) {
+foreach my $m (@$mods) {
 	my $mdir = &module_root_directory($m);
-	if ($m && &foreign_check($m) && !$_[4] &&
+	if ($m && &foreign_check($m) && !$show &&
 	    -r "$mdir/backup_config.pl") {
 		&foreign_require($m, "backup_config.pl");
 		if (&foreign_defined($m, "pre_restore")) {
@@ -576,7 +581,7 @@ foreach my $m (@{$_[0]}) {
 	}
 
 # Lock all files being extracted
-if (!$_[4]) {
+if (!$show) {
 	my $f;
 	foreach $f (@files) {
 		&lock_file($f);
@@ -584,7 +589,7 @@ if (!$_[4]) {
 	}
 
 # Extract contents (only files specified by manifests)
-my $flag = $_[4] ? "t" : "x";
+my $flag = $show ? "t" : "xv";
 my $qfiles = join(" ", map { s/^\///; quotemeta($_) } &unique(@files));
 if ($gzipped) {
 	&execute_command("cd / ; gunzip -c $qfile | tar ${flag}f - $qfiles",
@@ -597,7 +602,7 @@ else {
 my $ex = $?;
 
 # Un-lock all files being extracted
-if (!$_[4]) {
+if (!$show) {
 	my $f;
 	foreach $f (@files) {
 		&unlock_file($f);
@@ -610,16 +615,16 @@ if ($ex) {
 	return &text('backup_euntar', "<pre>$out</pre>");
 	}
 
-if ($_[3] && !$_[4]) {
+if ($apply && !$show) {
 	# Call all module apply functions
-	foreach $m (@{$_[0]}) {
+	foreach $m (@$mods) {
 		if (&foreign_defined($m, "post_restore")) {
 			&foreign_call($m, "post_restore", \@files);
 			}
 		}
 	}
 
-@{$_[2]} = @files;
+@$files = split(/\n/, $out);
 return undef;
 }
 
@@ -704,7 +709,7 @@ elsif ($mode == 4) {
 
 =head2 date_subs(string)
 
-Given a string with strftime-style format characters in it like %Y and %S, 
+Given a string with strftime-style format characters in it like %Y and %S,
 replaces them with the correct values for the current date and time.
 
 =cut
