@@ -32,7 +32,7 @@ $time_now = time();
 
 # Scan the entries in the log file
 &pre_process();
-open(LOG, $bandwidth_log);
+open(LOG, "<".$bandwidth_log);
 while(<LOG>) {
 	if (&process_line($_, \@hours, $time_now)) {
 		# Found a valid line
@@ -55,9 +55,11 @@ foreach $hour (@hours) {
 	&save_hour($hour);
 	}
 
-# Truncate the file and notify syslog
-open(LOG, ">$bandwidth_log");
-close(LOG);
+# Truncate the file (if it exists) and notify syslog
+if (-r $bandwidth_log) {
+	open(LOG, ">".$bandwidth_log);
+	close(LOG);
+	}
 &foreign_call($syslog_module, "signal_syslog");
 
 # Remove PID file
