@@ -20,7 +20,7 @@ return $rv;
 sub get_client_config
 {
 local $nis;
-open(CONF, $config{'client_conf'});
+open(CONF, "<".$config{'client_conf'});
 while(<CONF>) {
 	s/\r|\n//g;
 	s/#.*$//g;
@@ -121,7 +121,7 @@ foreach $d (@domains) {
 	print "<td valign=top><b>$text{'server_tables'}</b></td>\n";
 	print "<td><select name=tables_$n size=6 multiple>\n";
 	foreach $c (@conf) {
-		printf "<option value=%s %s>%s\n",
+		printf "<option value=%s %s>%s</option>\n",
 			$c->{'table'}, $c->{'active'} ? 'selected' : '',
 			$c->{'table'};
 		}
@@ -228,7 +228,7 @@ sub parse_nisupdate_conf
 {
 local @rv;
 local $lnum = 0;
-open(CONF, $_[0]);
+open(CONF, "<".$_[0]);
 while(<CONF>) {
 	s/\r|\n//g;
 	if (/^\s*(#*)(\s*\$rule{['"]([^"']+)['"]}.*)/) {
