@@ -19,11 +19,11 @@ return $rv;
 sub get_client_config
 {
 local $nis;
-open(DOM, "/etc/defaultdomain");
+open(DOM, "</etc/defaultdomain");
 chop($nis->{'domain'} = <DOM>);
 close(DOM);
 if ($nis->{'domain'}) {
-	if (open(SRV, "$binding_dir/$nis->{'domain'}/ypservers")) {
+	if (open(SRV, "<$binding_dir/$nis->{'domain'}/ypservers")) {
 		while(<SRV>) {
 			s/\r|\n//g;
 			push(@{$nis->{'servers'}}, $_);
@@ -137,7 +137,7 @@ map { $inall{$_}++ } @all;
 print "<tr> <td rowspan=4 valign=top><b>$text{'server_tables'}</b></td>\n";
 print "<td rowspan=4><select multiple size=6 name=tables>\n";
 foreach $t (&unique(@nis_tables, @all)) {
-	printf "<option value=%s %s>%s\n",
+	printf "<option value=%s %s>%s</option>\n",
 		$t, $inall{$t} ? 'selected' : '', $t;
 	}
 print "</select></td>\n";
@@ -238,7 +238,7 @@ sub parse_yp_makefile
 # First parse joined lines
 local $lnum = 0;
 local (@lines, $llast);
-open(MAKE, $yp_makefile);
+open(MAKE, "<".$yp_makefile);
 while(<MAKE>) {
 	s/\r|\n//g;
 	local $slash = (s/\\$//);
