@@ -157,7 +157,7 @@ map { $inall{$_}++ } split(/\s+/, $rule->{'all'}->{'value'});
 print "<tr> <td rowspan=2 valign=top><b>$text{'server_tables'}</b></td>\n";
 print "<td rowspan=2><select multiple size=5 name=tables>\n";
 foreach $t (grep { $rule->{$_} } @nis_tables) {
-	printf "<option value=%s %s>%s\n",
+	printf "<option value=%s %s>%s</option>\n",
 		$t, $inall{$t} ? 'selected' : '', $t;
 	}
 print "</select></td>\n";
@@ -171,7 +171,7 @@ printf "<td><input name=mingid size=10 value='%s'></td> </tr>\n",
 	$var->{'MINGID'}->{'value'};
 
 print "<tr> <td><b>$text{'server_slaves'}</b></td>\n";
-open(SLAVES, "/var/yp/ypservers");
+open(SLAVES, "</var/yp/ypservers");
 while(<SLAVES>) {
 	s/\s//g;
 	push(@slaves, $_) if ($_);
@@ -363,7 +363,7 @@ sub parse_rc_config
 local $rc;
 local $lnum = 0;
 local $f = $_[0] ? $_[0] : $rc_config;
-open(CONF, $f);
+open(CONF, "<".$f);
 while(<CONF>) {
 	s/\r|\n//g;
 	s/#.*$//g;
