@@ -49,7 +49,7 @@ if (opendir(DIR, $fn)) {
 else {
 	# Just a normal config file
 	local $lnum = 0;
-	if (open(CONF, $fn)) {
+	if (open(CONF, "<".$fn)) {
 		@rv = &parse_config_file(CONF, $lnum, $fn);
 		close(CONF);
 		foreach $inc (&find_directive("Include", \@rv)) {
@@ -80,6 +80,15 @@ $dummy{'file'} = $_[2];
 $dummy{'type'} = 0;
 $dummy{'name'} = "dummy";
 @rv = (\%dummy);
+local %defs;
+foreach my $d (&get_httpd_defines()) {
+        if ($d =~ /^(\S+)=(.*)$/) {
+                $defs{$1} = $2;
+                }
+        else {
+                $defs{$d} = '';
+                }
+        }
 while($line = <$fh>) {
 	chop;
 	$line =~ s/^\s*#.*$//g;
@@ -117,8 +126,8 @@ while($line = <$fh>) {
 		local $oldline = $_[1];
 		$_[1]++;
 		local @dirs = &parse_config_file($fh, $_[1], $_[2]);
-		if (!$not && $httpd_defines{$def} ||
-		    $not && !$httpd_defines{$def}) {
+		if (!$not && defined($defs{$def}) ||
+		    $not && !defined($defs{$def})) {
 			# use the directives..
 			push(@rv, { 'line', $oldline,
 				    'eline', $oldline,
@@ -218,7 +227,7 @@ return wantarray ? @vals : !@vals ? undef : $vals[$#vals];
 }
 
 # find_directive_struct(name, &directives)
-# Returns references to directives maching some name
+# Returns references to directives matching some name
 sub find_directive_struct
 {
 local(@rv, $i, @vals);
@@ -320,12 +329,12 @@ foreach $e (@{$_[0]}) {
 			print "<td colspan=2></td> </tr><tr>\n";
 			}
 		else { $sw = !$sw; }
-		print "<td valign=top align=right width=25%><b>$rv[1]</b></td>\n";
+		print "<td valign=top width=25%><b>$rv[1]</b></td>\n";
 		print "<td nowrap valign=top colspan=3 width=75%>$rv[2]</td>\n";
 		}
 	else {
 		# only spans one column
-		print "<td valign=top align=right width=25%><b>$rv[1]</b></td>\n";
+		print "<td valign=top width=25%><b>$rv[1]</b></td>\n";
 		print "<td nowrap valign=top width=25%>$rv[2]</td>\n";
 		}
 
@@ -427,7 +436,7 @@ local($i, $rv);
 $rv = "<select name=\"$_[1]\">\n";
 for($i=3; $i<@_; $i++) {
 	$_[$i] =~ /^([^,]*),(.*)$/;
-	$rv .= sprintf "<option value=\"$2\" %s> $1\n",
+	$rv .= sprintf "<option value=\"$2\" %s>$1</option>\n",
 		lc($2) eq lc($_[0]) || !defined($_[0]) && lc($2) eq lc($_[2]) ? "selected" : "";
 	}
 $rv .= "</select>\n";
@@ -607,7 +616,7 @@ return wantarray ? ($c, $v) : $c;
 sub get_ftpaccess_config
 {
 local($lnum, @conf);
-open(FTPACCESS, $_[0]);
+open(FTPACCESS, "<".$_[0]);
 @conf = &parse_config_file(FTPACCESS, $lnum, $_[0]);
 close(FTPACCESS);
 return \@conf;
@@ -670,7 +679,7 @@ if ($config{'test_always'}) {
 	local @files = &unique(map { $_->{'file'} } @$conf);
 	local $/ = undef;
 	foreach $f (@files) {
-		if (open(BEFORE, $f)) {
+		if (open(BEFORE, "<".$f)) {
 			$before_changing{$f} = <BEFORE>;
 			close(BEFORE);
 			}
@@ -790,9 +799,10 @@ else {
 	}
 }
 
+# get_proftpd_version([&output])
 sub get_proftpd_version
 {
-local $out = `$config{'proftpd_path'} -v 2>&1`;
+local $out = &backquote_command("$config{'proftpd_path'} -v 2>&1");
 ${$_[0]} = $out if ($_[0]);
 if ($out =~ /ProFTPD\s+Version\s+(\d+)\.([0-9\.]+)/i ||
     $out =~ /ProFTPD\s+(\d+)\.([0-9\.]+)/i) {
@@ -875,5 +885,59 @@ else {
 return $? ? "<pre>$out</pre>" : undef;
 }
 
+# get_httpd_defines()
+# Returns a list of defines that need to be passed to ProFTPd
+sub get_httpd_defines
+{
+if (@get_httpd_defines_cache) {
+	return @get_httpd_defines_cache;
+	}
+local @rv;
+if ($config{'defines_file'}) {
+	# Add defines from an environment file, which can be in
+	# the format :
+	# OPTIONS='-Dfoo -Dbar'
+	# or regular name=value format
+	local %def;
+	&read_env_file($config{'defines_file'}, \%def);
+	if ($config{'defines_name'} && $def{$config{'defines_name'}}) {
+		# Looking for var like OPTIONS='-Dfoo -Dbar'
+		local $var = $def{$config{'defines_name'}};
+		foreach my $v (split(/\s+/, $var)) {
+			if ($v =~ /^-[Dd](\S+)$/) {
+				push(@rv, $1);
+				}
+			else {
+				push(@rv, $v);
+				}
+			}
+		}
+	else {
+		# Looking for regular name=value directives.
+		# Remove $SUFFIX variable seen on debian that is computed
+		# dynamically, but is usually empty.
+		foreach my $k (keys %def) {
+			$def{$k} =~ s/\$SUFFIX//g;
+			push(@rv, $k."=".$def{$k});
+			}
+		}
+	}
+foreach my $md (split(/\t+/, $config{'defines_mods'})) {
+	# Add HAVE_ defines from modules
+	opendir(DIR, $md);
+	while(my $m = readdir(DIR)) {
+		if ($m =~ /^(mod_|lib)(.*).so$/i) {
+			push(@rv, "HAVE_".uc($2));
+			}
+		}
+	closedir(DIR);
+	}
+foreach my $d (split(/\s+/, $config{'defines'})) {
+	push(@rv, $d);
+	}
+@get_httpd_defines_cache = @rv;
+return @rv;
+}
+
 1;
 
