@@ -2,6 +2,8 @@
 # run-postinstalls.pl
 # Run all the postinstall.pl scripts in module and theme directories
 
+use lib '.';
+BEGIN { push(@INC, "."); };
 $no_acl_check++;
 use WebminCore;
 &init_config();
