@@ -61,7 +61,7 @@ local @rv = ( { 'dummy' => 1,
 		'line' => -1,
 		'eline' => -1 } );
 local $lnum = 0;
-open(CONF, $config{'sshd_config'});
+open(CONF, "<".$config{'sshd_config'});
 while(<CONF>) {
 	s/\r|\n//g;
 	s/^\s*#.*$//g;
@@ -181,7 +181,7 @@ local @rv = ( { 'dummy' => 1,
 		'eline' => -1 } );
 local $host;
 local $lnum = 0;
-open(CLIENT, $config{'client_config'});
+open(CLIENT, "<".$config{'client_config'});
 while(<CLIENT>) {
 	s/\r|\n//g;
 	s/^\s*#.*$//g;
@@ -322,7 +322,7 @@ else {
 
 # get_mlvalues(file, id, [splitchar])
 # Return an array with values from a file, where the
-# values are one per line with an id preceeding them
+# values are one per line with an id preceding them
 sub get_mlvalues
 {
 local @rv;
