@@ -6,6 +6,7 @@ $no_acl_check++;
 delete($ENV{'FOREIGN_MODULE_NAME'});
 delete($ENV{'SCRIPT_NAME'});
 delete($ENV{'SERVER_ROOT'});
+delete($ENV{'LANG'});
 require './status-lib.pl';
 
 # Check if the monitor should be run now
@@ -30,6 +31,8 @@ else {
 &read_file($oldstatus_file, \%oldstatus);
 &lock_file($fails_file);
 &read_file($fails_file, \%fails);
+&lock_file($lastsent_file);
+&read_file($lastsent_file, \%lastsent);
 
 # Get the list of services, ordered so that those with dependencies are first
 @services = &list_services();
@@ -129,7 +132,7 @@ foreach $serv (@services) {
 		if ($warn == 0 && $up == 0 && $o) {
 			# Service has just gone down
 			$suffix = "down";
-			$out = &run_on_command($serv, $serv->{'ondown'});
+			$out = &run_on_command($serv, $serv->{'ondown'}, $r);
 			}
 		elsif ($warn == 1 && $up != $o &&
 		       (defined($o) || $up == 0)) {
@@ -137,12 +140,12 @@ foreach $serv (@services) {
 			if ($up == 0) {
 				# A monitor has gone down
 				$suffix = "down";
-				$out = &run_on_command($serv, $serv->{'ondown'});
+				$out = &run_on_command($serv, $serv->{'ondown'}, $r);
 				}
 			elsif ($up == 1 && $o != -4) {
 				# A monitor has come back up after being down
 				$suffix = "up";
-				$out = &run_on_command($serv, $serv->{'onup'});
+				$out = &run_on_command($serv, $serv->{'onup'}, $r);
 				}
 			elsif ($up == -1) {
 				# Detected that a program the monitor depends on
@@ -157,40 +160,47 @@ foreach $serv (@services) {
 				# Monitor function timed out
 				$suffix = "timed";
 				$out = &run_on_command($serv,
-						       $serv->{'ontimeout'});
+						       $serv->{'ontimeout'}, $r);
 				}
 			}
 		elsif ($warn == 2 && $up == 0) {
 			# Service is down now
 			$suffix = "isdown";
-			$out = &run_on_command($serv, $serv->{'ondown'});
+			$out = &run_on_command($serv, $serv->{'ondown'}, $r);
+			}
+		elsif ($warn == 3 && $up == 1) {
+			# Service is up now
+			$suffix = "isup";
+			$out = &run_on_command($serv, $serv->{'onup'}, $r);
 			}
 
 		# If something happened, notify people
-		if ($suffix) {
+		if ($suffix &&
+		    $nowunix - $lastsent{$serv->{'id'}} > $config{'email_interval'} * 60) {
 			$subj = &text('monitor_sub_'.$suffix,
 				      $serv->{'desc'}, $host);
 			if ($notify{'pager'}) {
 				$pager_msg .= &make_message($suffix, $host,
-							    $serv, 'pager');
+							$serv, 'pager', $stat);
 				}
 			if ($notify{'sms'}) {
 				$sms_msg .= &make_message($suffix, $host,
-                                                          $serv, 'sms');
+							$serv, 'sms', $stat);
 				}
 			if ($notify{'snmp'}) {
 				push(@snmp_msg, &make_message($suffix, $host,
-                                                              $serv, 'snmp'));
+							$serv, 'snmp', $stat));
 				}
 			if ($notify{'email'}) {
 				$thisemail .= &make_message($suffix, $host,
-                                                              $serv, 'email');
+							$serv, 'email', $stat);
 				if ($out) {
 					$thisemail .= $out;
 					}
 				$thisemail .= "\n";
 				$ecount++;
 				}
+			$lastsent{$serv->{'id'}} = $nowunix;
 			}
 		$newstats->{$r} = $up;
 		$newvalues->{$r} = $stat->{'value'};
@@ -244,13 +254,20 @@ foreach $serv (@services) {
 
 	# Update old status hash
 	$oldstatus{$serv->{'id'}} = $newstatus_str;
+
+	# If successful, clear the last-sent time
+	if ($ok == 1) {
+		delete($lastsent{$serv->{'id'}});
+		}
 	}
 
-# Close oldstatus and fails files
+# Close oldstatus, fails and lastsent files
 &write_file($oldstatus_file, \%oldstatus);
 &unlock_file($oldstatus_file);
 &write_file($fails_file, \%fails);
 &unlock_file($fails_file);
+&write_file($lastsent_file, \%lastsent);
+&unlock_file($lastsent_file);
 
 # Send the email and page with all messages, if necessary
 if ($ecount && !$config{'sched_single'}) {
@@ -304,7 +321,12 @@ local ($carrier) = grep { $_->{'id'} eq $config{'sched_carrier'} }
 			&list_sms_carriers();
 return if (!$carrier);
 local $email = $config{'sched_sms'}."\@".$carrier->{'domain'};
-&mailboxes::send_text_mail($from, $email, undef, undef, $text,
+local $subject = $config{'sched_subject'};
+if ($subject eq "*") {
+	$subject = $text;
+	$text = undef;
+	}
+&mailboxes::send_text_mail($from, $email, undef, $subject, $text,
 			   $config{'sched_smtp'});
 }
 
@@ -401,34 +423,35 @@ if (!$@) {
 print STDERR "No SNMP perl module found\n";
 }
 
-# run_on_command(&serv, command)
+# run_on_command(&serv, command, remote-host)
 sub run_on_command
 {
-local ($serv, $cmd) = @_;
+local ($serv, $cmd, $r) = @_;
+$r = undef if ($r eq "*");
 return undef if (!$cmd);
 local $out;
-if ($serv->{'runon'} && $serv->{'remote'}) {
+if ($serv->{'runon'} && $r) {
 	# Run on the remote host
 	$remote_error_msg = undef;
-	&remote_foreign_call($serv->{'remote'}, "status",
+	&remote_foreign_call($r, "status",
 		"set_monitor_environment", $serv);
 	&remote_error_setup(\&remote_error_callback);
 	if ($config{'output'}) {
-		$out = &remote_foreign_call($serv->{'remote'}, "status",
+		$out = &remote_foreign_call($r, "status",
 			"backquote_command", "($cmd) 2>&1 </dev/null");
 		}
 	else {
-		&remote_foreign_call($serv->{'remote'}, "status",
+		&remote_foreign_call($r, "status",
 			"execute_command", $cmd);
 		}
 	&remote_error_setup(undef);
-	&remote_foreign_call($serv->{'remote'}, "status",
+	&remote_foreign_call($r, "status",
 		"reset_monitor_environment", $serv);
 	if ($remote_error_msg) {
-		return &text('monitor_runerr', $cmd, $serv->{'remote'},
+		return &text('monitor_runerr', $cmd, $r,
 			     $remote_error_msg);
 		}
-	return &text('monitor_run1', $cmd, $serv->{'remote'})."\n".$out;
+	return &text('monitor_run1', $cmd, $r)."\n".$out;
 	}
 else {
 	# Just run locally
@@ -480,17 +503,17 @@ $t =~ s/([=\177-\377])/sprintf("=%2.2X",ord($1))/ge;
 return $t;
 }
 
-# make_message(status, host, &server, type)
+# make_message(status, host, &server, type, &status)
 # Returns the message for some email, SMS or SNMP. May use a template, or
 # the built-in default.
 sub make_message
 {
-local ($suffix, $host, $serv, $type) = @_;
+local ($suffix, $host, $serv, $type, $stat) = @_;
 local $tmpl = $serv->{'tmpl'} ? &get_template($serv->{'tmpl'}) : undef;
 if ($tmpl && $tmpl->{$type}) {
 	# Construct from template
 	local %hash = ( 'DESC' => $serv->{'desc'},
-			'HOST' => $host,
+			'HOST' => $host || &get_system_hostname(),
 			'DATE' => $nowdate,
 			'TIME' => $nowtime,
 			'STATUS' => $text{'mon_'.$suffix},
@@ -499,6 +522,11 @@ if ($tmpl && $tmpl->{$type}) {
 	foreach my $s (@monitor_statuses) {
 		$hash{uc($s)} ||= 0;
 		}
+	if ($stat) {
+		foreach my $k ('value', 'nice_value', 'desc') {
+			$hash{'STATUS_'.uc($k)} = $stat->{$k} ? $stat->{$k} : "";
+			}
+		}
 	foreach my $k (keys %$serv) {
 		$hash{'SERVICE_'.uc($k)} = $serv->{$k};
 		}
@@ -522,8 +550,13 @@ else {
 			     $host, $serv->{'desc'});
 		}
 	elsif ($type eq 'email') {
-		return &text('monitor_email_'.$suffix,
-			     $host, $serv->{'desc'}, $now)."\n";
+		my $rv = &text('monitor_email_'.$suffix,
+			       $host, $serv->{'desc'}, $now)."\n";
+		if ($stat->{'desc'}) {
+			$rv .= &text('monitor_email_stat',
+				     $stat->{'desc'})."\n";
+			}
+		return $rv;
 		}
 	}
 }
