@@ -2,8 +2,9 @@
 # Collect various pieces of general system information, for display by themes
 # on their status pages. Run every 5 mins from Cron.
 
+use strict;
+use warnings;
 package system_status;
-$main::no_acl_check++;
 require './system-status-lib.pl';
 &scheduled_collect_system_info();
 
