@@ -1,13 +1,16 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
+use strict;
+use warnings;
+our (%text);
 do 'webalizer-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
 sub parse_webmin_log
 {
-local ($user, $script, $action, $type, $object, $p) = @_;
+my ($user, $script, $action, $type, $object, $p) = @_;
 if ($type eq "log") {
 	return &text("log_${action}_log", "<tt>".&html_escape($object)."</tt>");
 	}
